# w.c.s. - web application for online forms
# Copyright (C) 2005-2020  Entr'ouvert
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, see <http://www.gnu.org/licenses/>.

import xml.etree.ElementTree as ET

from quixote import get_publisher

from wcs.qommon.misc import xml_node_text
from wcs.qommon.storage import Equal, Or
from wcs.qommon.xml_storage import XmlStorableObject


class ApiAccess(XmlStorableObject):
    _names = 'apiaccess'
    xml_root_node = 'apiaccess'
    name = None
    access_identifier = None
    access_key = None
    description = None
    restrict_to_anonymised_data = False
    roles = None

    # declarations for serialization
    XML_NODES = [
        ('name', 'str'),
        ('description', 'str'),
        ('access_identifier', 'str'),
        ('access_key', 'str'),
        ('restrict_to_anonymised_data', 'bool'),
        ('roles', 'roles'),
    ]

    @classmethod
    def get_by_identifier(cls, access_identifier):
        for api_access in cls.select():
            if api_access.access_identifier == access_identifier:
                return api_access
        return None

    @classmethod
    def get_access_key(cls, access_identifier):
        api_access = cls.get_by_identifier(access_identifier)
        if api_access:
            return api_access.access_key
        return None

    def get_roles(self):
        return self.roles or []

    def export_roles_to_xml(self, element, attribute_name, include_id=False, **kwargs):
        for role in self.get_roles():
            sub = ET.SubElement(element, 'role')
            if include_id:
                sub.attrib['role-id'] = role.id
            sub.attrib['role-slug'] = role.slug
            sub.text = role.name

    def import_roles_from_xml(self, element, include_id=False, **kwargs):
        criterias = []
        for sub in element:
            if include_id and 'role-id' in sub.attrib:
                criterias.append(Equal('id', sub.attrib['role-id']))
            elif 'role-slug' in sub.attrib:
                criterias.append(Equal('slug', sub.attrib['role-slug']))
            else:
                role_name = xml_node_text(sub)
                if role_name:
                    criterias.append(Equal('name', role_name))
        return get_publisher().role_class.select([Or(criterias)], order_by='name')

    def get_as_api_user(self):
        class RestrictedApiUser:
            # kept as inner class so cannot be pickled
            id = Ellipsis  # make sure it fails all over the place if used
            is_admin = False
            is_api_user = True

            def can_go_in_backoffice(self):
                return False

            def get_roles(self):
                return self.roles

        user = RestrictedApiUser()
        user.roles = [x.id for x in self.get_roles()]
        return user

    @classmethod
    def get_with_credentials(cls, username, password):
        api_access = cls.get_by_identifier(username)
        if not api_access or api_access.access_key != password:
            raise KeyError
        return api_access.get_as_api_user()
